classdef Scenario < handle 
    
    properties
        NofCells
        NofCellsX
        NofCellsY
        Duration
        Strategies
        Noperators
        Nservices
         % area parameters
        AreaSize
        CellSize
        simTime
    end
    
    properties (SetAccess=private)
        % newEvent funtion
        newEvent
        
        % distribution functions
        genPositionDist
        genOperatorDist
        genServiceDist
                
        Users
        
        firstUser  
        lastUser
    end
    
    methods
        function S = Scenario(duration, newEvent, genPositionDist, genOperatorDist, Noperators, genServiceDist, AreaSize, CellSize, Strategies)
            S.newEvent=newEvent;
            S.genPositionDist=genPositionDist;
            S.genOperatorDist=genOperatorDist;
            S.Noperators = Noperators;            
            S.genServiceDist = genServiceDist;
            S.AreaSize=AreaSize;
            S.CellSize=CellSize;
            S.NofCellsX=S.AreaSize.x/S.CellSize.x;
            S.NofCellsY=S.AreaSize.y/S.CellSize.y;
            S.NofCells=S.NofCellsX*S.NofCellsY;
            S.Duration = duration;
            S.Strategies = Strategies;
            S.simTime=0;
            S.Nservices=genServiceDist();
            S.firstUser=zeros(length(S.Strategies),1);
            S.lastUser=zeros(length(S.Strategies),1);
            for i=1:length(S.Strategies)
                S.Strategies(i).config(S,i);
            end;
               
        end        
        function user=newUser(S,N)
            if nargin==1
                N=1;
            end
            user(N) = User;
            [events time]=S.newEvent(N,S.simTime);
            Nnew=sum(events==1);
            [x y] = S.genPositionDist(S,Nnew);
            x=min(S.AreaSize.x,x);
            x=max(0.01,x);
            y=min(S.AreaSize.y,y);
            y=max(0.01,y);            
            if Nnew>0
                operators = S.genOperatorDist(S,[x y]);
                services = S.genServiceDist(S,[x y],operators);
            end
            k=1;
            for i=1:N
                user(i).Event=events(i);
                user(i).time=time(i);                
                if (user(i).Event == 1)
                    user(i).Position.x=x(k);
                    user(i).Position.y=y(k);
                    user(i).Operator = operators(k);
                    user(i).Service = services(k);
                    k=k+1;
                end
            end;
            S.Users=[S.Users user];
        end
        
        function cell=getCell(S,User)
            cell(length(User))=Cell;
            pos=[User.Position];
            X=ceil([pos.x]./S.CellSize.x);
            Y=ceil([pos.y]./S.CellSize.y);    
            Idx=(Y-1)*S.NofCellsX+X;
            Rel_X=X./S.NofCellsX;
            Rel_Y=Y./S.NofCellsY;
            for i=1:length(User)
                cell(i).X=X(i);
                cell(i).Y=Y(i);
                cell(i).Idx=Idx(i);
                cell(i).Rel_Idx=Idx(i)/S.NofCells;
                if (cell(i).Rel_Idx>1)
                    k=0;
                end
                cell(i).Rel_X=Rel_X(i);
                cell(i).Rel_Y=Rel_Y(i);
            end
        end
        
        
        function [usersX usersY colors]=getUsersXY(S,operator,service,strategy)
            if (nargin==1)
                operator=0;
                service=0;
                strategy=0;
            elseif (nargin==2)
                service=0;
                strategy=0;
            elseif (nargin==3)
                strategy=0;
            end;
            
            if strategy>0
                us=S.Strategies(strategy).listFeasibles(operator,service);
            else                
                us=[S.Users];
                us=us(filterOpServ(us,operator,service));                            
            end            
            pos=[us.Position];
            usersX=[pos.x];
            usersY=[pos.y];
            colors=[us.Operator];
        end
                
        function run(S,Nsteps)
            % generate a new user for this scenario    
            user = S.newUser(Nsteps);
            newUsers=user(user.isNew());
            
             % get the user cell location in this scenario
             if ~isempty(newUsers)
                 cell = S.getCell(newUsers);

                 Res(length(newUsers))=MappingResult;
             else
                 Res=[];
             end
             
             
             
             % for all strategies
             for j=1:length(S.Strategies)         
                 % get clusters for this strategy
                 if ~isempty(newUsers)
                    cluster = S.Strategies(j).getCluster(cell,newUsers);
                 end
                 k=1;
                 for n=1:Nsteps
                    % Add or remove the user to/from the cluster
                    if user(n).isNew()
                        if (length(user(n).Result)==1)
                            user(n).Result(length(S.Strategies))=MappingResult;
                            user(n).cluster(length(S.Strategies))=Cluster;
                        end;
                        
                        Res(k) = S.Strategies(j).mapUser(cell(k), cluster(k), user(n));   
                        if (Res(k).isFeasible())
                            if  S.firstUser(j)==0
                                S.firstUser(j)=S.simTime+n;
                            end;
                            user(n).cluster(j,:)=cluster(k);
                            user(n).cell=cell(k);     
                            user(n).Result(j)=Res(k);
                            
                        end;
                        S.lastUser(j)=S.simTime+n;
                        k=k+1;
                    else
                        % unmap and remove from cell, allways possible         
                        if (S.firstUser(j)<S.lastUser(j))
                            u=S.Users(S.firstUser(j));        
                            
                            %%TODO
                            if (u.Result(j).isFeasible())
                                S.Strategies(j).unmapUser(u.cell, u.cluster(j), u.Result(j), u);           
                            end
                            for i=S.firstUser(j)+1:S.lastUser(j)
                            	if S.Users(i).isNew();
                                    if (S.Users(i).Result(j).isFeasible())
                                        break;
                                    end;
                                end
                            end;
                            S.firstUser(j)=i;
                        end;
                    end;                         
                     S.Strategies(j).Cloud.updateTime(S.simTime+n);                       
                 end        
            
                 if ~isempty(Res)
                     S.Strategies(j).SaveResult(Res,user);
                 end;
            end; 

            S.simTime=S.simTime+Nsteps;
            
        end
        

    end
    
    
end