classdef Strategy < handle

    properties (SetAccess=private)
        Cells  
        Scenario          
        idx
        
        feasiblesMap
        unfeasiblesMap        
        numUsersTime
        numFeasiblesTime
        numActiveProcessorsTime
        meanusageProcessorsTime
        usageProcessorsVectorTime
        currentIdx
        savedIdx
    end
    properties
        mappingConf
        customMap
        HLmapping
        HLargs
        Nclusters
        clustSize;
        
        Cloud
        results = MappingResult
        users = User        
        time
        simTime
    end
        
    methods
        %% main functions
        function S = Strategy(mappingConf,Cloud,HLmapping,HLargs)
            S.mappingConf=mappingConf;
            S.Nclusters=Cloud.NC;
            S.clustSize=Cloud.ClustSize;
            S.Cloud = Cloud;          
            S.HLmapping=HLmapping;
            % Init hlmapping
            HLmapping();
            S.HLargs=HLargs;
            S.simTime=1;
            mappingConf.setCloud(Cloud);
        end
        function config(S, Scenario,idx)
            S.users(Scenario.Duration)=User;
            S.results(Scenario.Duration)=MappingResult;
            S.time=zeros(Scenario.Duration,1);
            S.currentIdx=1;
            
            S.feasiblesMap=zeros(Scenario.NofCellsX,Scenario.NofCellsY,Scenario.Noperators,Scenario.Nservices);
            S.unfeasiblesMap=zeros(Scenario.NofCellsX,Scenario.NofCellsY,Scenario.Noperators,Scenario.Nservices);
            S.Scenario=Scenario;            
           
            S.numUsersTime=zeros(Scenario.Duration,1);
            S.numFeasiblesTime=zeros(Scenario.Duration,1);
            S.numActiveProcessorsTime=zeros(Scenario.Duration,1);
            S.meanusageProcessorsTime=zeros(Scenario.Duration,1);
            S.usageProcessorsVectorTime=zeros(S.Cloud.W,S.Cloud.N);
            S.idx=idx;
            S.savedIdx=1;
        end
                    
        function cluster = getCluster(S, cell, user)      
            cluster(length(user))=Cluster;
            for i=1:length(user)
                % returns an array of node indexes
                id = S.HLmapping(S.Cloud,cell(i),user(i),S.HLargs);
                cluster(i).Idx=id;
            end
        end
        
        function unmapUser(S, cell, cluster, Res, user)
            Cc=S.Cloud.C(cluster.Idx);            
            for k=1:length(user.Service.c)
                p=Res.result(k)+1;
                % remove C resources
                Cc(p)=Cc(p)+user.Service.c(k);
                % remove B resources
               %%TODO 
            end
            if (Res.isFeasible())
                S.feasiblesMap(cell.X,cell.Y,user.Operator,user.Service.idx) = S.feasiblesMap(cell.X,cell.Y,user.Operator,user.Service.idx) - 1;
            else    
                S.unfeasiblesMap(cell.X,cell.Y,user.Operator,user.Service.idx) = S.unfeasiblesMap(cell.X,cell.Y,user.Operator) - 1;                
            end
            S.Cloud.C(cluster.Idx)=Cc;
            S.simTime=S.simTime+1;
        end
        
        function R = mapUser(S, cell, cluster, user)
            R = S.mappingConf.map(cluster.Idx,user);
            S.simTime=S.simTime+1;
            if (R.isFeasible())
                S.feasiblesMap(cell.X,cell.Y,user.Operator,user.Service.idx) = S.feasiblesMap(cell.X,cell.Y,user.Operator,user.Service.idx) + 1;
            else    
                S.unfeasiblesMap(cell.X,cell.Y,user.Operator,user.Service.idx) = S.unfeasiblesMap(cell.X,cell.Y,user.Operator) + 1;
            end        
        end
        
        function SaveResult(S, res, user)
            sidx=S.savedIdx:S.savedIdx+length(user)-1;
            S.users(sidx)=user;
            
            neIdx=user.isNew();
            tmp(length(user))=MappingResult;
            tmp(neIdx)=res;
            S.results(sidx)=tmp;

            exIdx=user.isExit();
            fesIdx=res.isFeasible();            
            if (S.savedIdx==1)
                sidx2=2:length(user)+1;
            else
                sidx2=sidx;
            end;
            j=1;
            p=1;
            for i=sidx2
                if exIdx(i-sidx2(1)+1)
                    if (S.numUsersTime(i-1)>0)
                        S.numUsersTime(i)=S.numUsersTime(i-1)-1;
                    end;
                    if (S.numFeasiblesTime(i-1)>0)
                        S.numFeasiblesTime(i)=S.numFeasiblesTime(i-1)-1;
                    end;
                elseif neIdx(i-sidx2(1)+1)
                    S.numUsersTime(i)=S.numUsersTime(i-1)+1;
                    if fesIdx(j)
                        S.numFeasiblesTime(i)=S.numFeasiblesTime(i-1)+1;
                    else
                        S.numFeasiblesTime(i)=S.numFeasiblesTime(i-1);
                    end;
                    j=j+1;
                else
                    S.numFeasiblesTime(i)=S.numFeasiblesTime(i-1);
                    S.numUsersTime(i)=S.numUsersTime(i-1);
                end;
                if S.numFeasiblesTime(i)<S.numFeasiblesTime(i-1)-4
                    p=0;
                end;
            end;
            
            
            
            % save processors state
            S.numActiveProcessorsTime(sidx)=S.numUsedProcs(sidx);
            S.meanusageProcessorsTime(sidx)=S.Cloud.usageProcs(sidx);
            sidx2=mod(sidx,S.Cloud.W);
            sidx2(sidx2==0)=1;
            S.usageProcessorsVectorTime(sidx2,:)=S.Cloud.usageProcsVect(sidx2);
           
            S.time(sidx)=cumsum([user.time]);
            if (S.savedIdx>1)
                S.time(sidx)=S.time(sidx)+S.time(S.savedIdx-1);
            end;
            
            S.savedIdx=S.savedIdx+length(user);
            S.currentIdx=1:S.savedIdx-1;
                     
        end
        
        %% users lists
        function users=listFeasibles(S,operator,service)
            res=S.results(S.currentIdx);
            users=S.users(res.isFeasible());
            users=users(filterOpServ(users,operator,service));
        end
        function users=listUnfeasibles(S,operator,service)
            res=S.results(S.currentIdx);
            users=S.users(res.isUnfeasible());
            users=users(filterOpServ(users,operator,service));
        end
        function users=listNew(S,operator,service)
            users=S.users(S.currentIdx);
            users=users.isNew();
            users=users(filterOpServ(users,operator,service));
        end
        function users=listExit(S,operator,service)
            users=S.users(S.currentIdx);
            users=users.isExit();
            users=users(filterOpServ(users,operator,service));
        end       
        function users=listAll(S,operator,service)
            users=S.users(S.currentIdx);
            users=users(filterOpServ(users,operator,service));
        end
        
        %% Users MAPS        
        function map=UnfeasibleUsersMap(S, operator, service)
            if (nargin==1 || (operator==0 && service==0))
                n=size(S.unfeasiblesMap);   
                map=zeros(S.Scenario.NofCellsX,S.Scenario.NofCellsY);
                for i=1:n(3)
                    for j=1:n(4)
                        map=map+S.unfeasiblesMap(:,:,i,j);
                    end;
                end;
            elseif (nargin==2 || (operator>0 && service==0))
                n=size(S.unfeasiblesMap);   
                map=zeros(S.Scenario.NofCellsX,S.Scenario.NofCellsY);
                for j=1:n(4)
                    map=map+S.unfeasiblesMap(:,:,operator,j);
                end;                
            elseif (nargin==3 && (operator==0 && service>0))
                n=size(S.unfeasiblesMap);   
                map=zeros(S.Scenario.NofCellsX,S.Scenario.NofCellsY);
                for j=1:n(3)
                    map=map+S.unfeasiblesMap(:,:,j,service);
                end;                
            elseif (nargin==3 && (operator>0 && service>0))
                map=S.unfeasiblesMap(:,:,operator,service);
            end
        end        
        function map=FeasibleUsersMap(S, operator, service)
            if (nargin==1 || (operator==0 && service==0))
                n=size(S.feasiblesMap);   
                map=zeros(S.Scenario.NofCellsX,S.Scenario.NofCellsY);
                for i=1:n(3)
                    for j=1:n(4)
                        map=map+S.feasiblesMap(:,:,i,j);
                    end;
                end;
            elseif (nargin==2 || (operator>0 && service==0))
                n=size(S.feasiblesMap);   
                map=zeros(S.Scenario.NofCellsX,S.Scenario.NofCellsY);
                for j=1:n(4)
                    map=map+S.feasiblesMap(:,:,operator,j);
                end;                
            elseif (nargin==3 && (operator==0 && service>0))
                n=size(S.feasiblesMap);   
                map=zeros(S.Scenario.NofCellsX,S.Scenario.NofCellsY);
                for j=1:n(3)
                    map=map+S.feasiblesMap(:,:,j,service);
                end;                
            elseif (nargin==3 && (operator>0 && service>0))
                map=S.feasiblesMap(:,:,operator,service);
            end
        end         
        function map=UsersMap(S,operator,service)
            map=S.UnfeasibleUsersMap(operator,service)+S.FeasibleUsersMap(operator,service);
        end

        %% Users Numbers        
        function Num=numFeasibles(S, operator, service)
            if (nargin==1)
                operator=0;
                service=0;
            elseif (nargin==2)
                service=0;
            end;
            Num=length(S.listFeasibles(operator,service));
        end
        function Num=numUnfeasibles(S, operator, service)
            if (nargin==1)
                operator=0;
                service=0;
            elseif (nargin==2)
                service=0;
            end;
            Num=length(S.listUnfeasibles(operator,service));
        end
        function Num=numUsers(S, operator, service)
            if (nargin==1)
                operator=0;
                service=0;
            elseif (nargin==2)
                service=0;
            end;
            Num=S.numFeasibles(operator,service)+S.numUnfeasibles(operator,service);
        end
        
        %% Users vectors
        function [v t]=vectorFeasibles(S)                      
            v=S.numFeasiblesTime(S.currentIdx);
            t=S.time(S.currentIdx);            
        end        
        function [v t]=vectorUsers(S)
            v=S.numUsersTime(S.currentIdx);
            t=S.time(S.currentIdx);            
        end

        %% Processors vectors
        function [v t]=vectorActiveProcessors(S)
            v=S.numActiveProcessorsTime(S.currentIdx);
            t=S.time(S.currentIdx);            
        end        
        function [v t]=vectorUsageProcessors(S)
            v=S.meanusageProcessorsTime(S.currentIdx);
            t=S.time(S.currentIdx);            
        end        
        function [v t]=usageProcessorsTime(S)
            v=S.usageProcessorsVectorTime(S.currentIdx,:);
            t=S.time(S.currentIdx);            
        end
                        
        %% Processors numbers
        function procs=numUsedProcs(S,time)
            if nargin==1
                time=S.simTime;                
            end;
            if (time>S.Scenario.Duration)
                time=S.Scenario.Duration;
            end;
            procs = S.Cloud.numUsedProcs(time);
        end
        
        %% Processors map           
        function x=usageProcessorsMap(S)
            x=S.Cloud.usageProcsMap();
        end
        function x=usageBWMap(S)
            x=S.Cloud.usageBWMap();
        end     
        
        %% Mapping time
        function t=mappingTime(S,operator,service)                
            if (nargin==1)
                operator=0;
                service=0;
            elseif (nargin==2)
                service=0;
            end;
            if (~isempty(S.results))
                a=S.listAll(0,0);
                idx=filterOpServ(a,operator,service);
                res=S.results(idx);
                t=[res.time];
            else
                t=[];
            end
        end
        
    end
    
            
        
    
end
