clear;

% SCENARIO: Determines the randomness of the simulation, e.g.: size of the
% area, number of sub-areas, number of users, distributions, etc.

% STRATEGY: For each different scenario, different strategies can be
% evaluated. The random realization is equal for each scenario. 

% read array of scenarios
S = readScenarios();

% for all scenarios
for i=1:length(S)


    % generate a new user for this scenario    
    user = S(i).newUser(S(i).Duration);
    
     % get the user cell location in this scenario
     cell = S(i).getCell(user);

     Res(S(i).Duration)=MappingResult;

     % for all strategies
     for j=1:length(S(i).Strategies)         
         % get clusters for this strategy
         cluster = S(i).Strategies(j).getCluster(cell,user);

         
         for n=1:S(i).Duration
            % Add or remove the user to/from the cluster
            if user(n).isNew()
                Res(n) = S(i).Strategies(j).mapUser(cell(n), cluster(n), user(n));                                        
            else
                % unmap and remove from cell, allways possible                   
                S(i).Stragegies(j).unmapUser(cell(n), user(n));
            end;                         
         end        
         
         S(i).Strategies(j).SaveResult(Res,user);
    end;

    %print results
%     figure;
%     [x y c] = S(i).getUsersXY();
%     scatter(x,y,5,c);
    fprintf('* Scenary %d\n',i);            
    for j=1:length(S(i).Strategies)
%         figure;
        Nops=S(i).Noperators;
        fprintf('\t* Strategy %d\n',j);
        for o=1:Nops
%             subplot(2,Nops,(o-1)*2+1)
%             p=S(i).Strategies(j).UnfeasibleUsersMapOperator(o)./S(i).Strategies(j).UsersMapOperator(o);
%             p(isnan(p))=0;
%             surf(p);
%             subplot(2,Nops,(o-1)*2+2)
%             surf(S(i).Strategies(j).UsersMapOperator(o));            
            fprintf('\t\t- Operator %d: Total Users=%d (%.2f%% feasibles)\n',o,S(i).Strategies(j).numUsersOperator(o),100*(S(i).Strategies(j).numFeasiblesOperator(o)/S(i).Strategies(j).numUsersOperator(o)));
        end;
    end;
end;
