% Cx, Bx: row vectors
% Ncluster: 2, 4, 9, 16, 25, 36, 49, ....power of 2

function [C B I] = torus(C_c,B_c,I_c,Ncluster,B_torus)

%B_torus = 100000;    % Mbps of the inter-cluster connections

Nrows = sqrt(Ncluster);     % Number of rows in the Torus (extended 2D Mesh)

if (Ncluster < 9)
    error('The number of clusters needs to be 9 at least\n');
end;

if (((floor(Nrows))^2)<Ncluster)
    error('The number of clusters needs to be a quadratic value, e.g. 16, 64, ..\n');
end;

N_proc_per_cluster = size(C_c,2);

inf = 10000000;
% Access to the intercluster communication network
O_c = ones(N_proc_per_cluster); % all processors have access to the inter-cluster communication network.

inter_proc = 4; % option '4' means manual (%nivel CLUSTER)
inter_clust=2;  % todos con todos mediante un solo bus:1  manual:2 (%nivel DATA CENTER)

% Data center at cluster level/granularity:
Ctotc = sum(C_c);    % MOPS of a single cluster

% Initializations:
C_d = Ctotc*ones(1,Ncluster);
I_d = (Ncluster*Ncluster).*ones(Ncluster,Ncluster);   % points to the last element (zero-link)
B_d = zeros(1,Ncluster*Ncluster);

for i=1:Ncluster
    I_d(i,i)=i;
    B_d(i) = inf;   % infinito, pero quizás debe ser la capacidad de los links internos?
end

% Torus ("extended 2D mesh")
% Clusters are numbered row by row:
%   1           2       3     ...    Nrows
% (Nrows+1)   (Nrows+2)       ...    Nrows+Nrows
% (2*Nrows+1) (2*Nrows+2)     ...    (2*Nrows+Nrows)
% ...
% (Nrows-1)*Nrows+1    ...            (Nrows-1)*Nrows+Nrows

count = Ncluster+1;
% horizontal connections
for j=1:Nrows   % row
    for i=1:(Nrows-1)   % column
        I_d((j-1)*Nrows+i,(j-1)*Nrows+i+1) = count;
        I_d((j-1)*Nrows+i+1,(j-1)*Nrows+i) = count;   % Bidireccional
        B_d(count) = B_torus;
        count = count+1;
    end
    % last-to-first cluster
    I_d(j*Nrows,(j-1)*Nrows+1) = count;
    I_d((j-1)*Nrows+1,j*Nrows) = count;     % Bidireccional
    B_d(count) = B_torus;
    count = count+1;
end


% verical connections
for i=1:Nrows   % column
    for j=1:(Nrows-1)   % row
        I_d((j-1)*Nrows+i,j*Nrows+i) = count;
        I_d(j*Nrows+i,(j-1)*Nrows+i) = count;   % Bidireccional
        B_d(count) = B_torus;
        count = count+1;
    end
    % last-to-first cluster
    I_d((Nrows-1)*Nrows+i,i) = count;
    I_d(i,(Nrows-1)*Nrows+i) = count;   % Bidireccional
    B_d(count) = B_torus;
    count = count+1;
end

[I_d B_d] = trans(I_d,B_d,Ncluster);

% Default parameters (no need to change here)
B_proc=20;
B_clust=50;

num_procesadores = N_proc_per_cluster*Ncluster;

[C I B] = generateDCmodelV(C_d,I_d,B_d,C_c,I_c,B_c,O_c,num_procesadores,inter_proc,B_proc,Ncluster,inter_clust,B_clust);

